@@ -21,9 +21,11 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: export.c,v 1.17 2003/10/21 20:53:05 mschimek Exp $ */
+/* $Id: export.c,v 1.21 2005/01/20 01:40:14 mschimek Exp $ */
 
-#include "../config.h"
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -37,7 +39,7 @@
 #include "export.h"
 #include "vbi.h" /* vbi_asprintf */
 
-const char _zvbi_intl_domainname[] = PACKAGE;
+extern const char _zvbi_intl_domainname[];
 
 /**
  * @addtogroup Export Exporting formatted Teletext and Closed Caption pages
@@ -525,7 +527,7 @@ option_string(vbi_export *e, const char *s2)
 vbi_export *
 vbi_export_new(const char *keyword, char **errstr)
 {
-	unsigned char key[256];
+	char key[256];
 	vbi_export_class *xc;
 	vbi_export *e;
 	unsigned int keylen;
@@ -736,7 +738,7 @@ vbi_export_option_get(vbi_export *export, const char *keyword,
 /**
  * @param export Pointer to a initialized vbi_export object.
  * @param keyword Keyword identifying the option, as in vbi_option_info.
- * @param Varargs New value to set.
+ * @param ... New value to set.
  *
  * Sets the value of the named option. Make sure the value is casted
  * to the correct type (int, double, char *).
@@ -986,8 +988,9 @@ vbi_export_file(vbi_export *export, const char *name,
 	reset_error(export);
 
 	if (!(fp = fopen(name, "w"))) {
-		vbi_export_error_printf(export, _("Cannot create file '%s': Error %d, %s."),
-					name, errno, strerror(errno));
+		vbi_export_error_printf(export,
+					_("Cannot create file '%s': %s."),
+					name, strerror(errno));
 		return FALSE;
 	}
 
