@@ -22,10 +22,13 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: vbi.c,v 1.12 2003/11/14 05:29:18 mschimek Exp $ */
+/* $Id: vbi.c,v 1.15 2004/12/13 07:11:12 mschimek Exp $ */
 
 #include "site_def.h"
-#include "../config.h"
+
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -54,6 +57,7 @@
  *
  * @author Iaki Garca Etxebarria<br>
  * Michael H. Schimek<br>
+ * Tom Zoerner<br>
  * based on AleVT by Edgar Toernig
  *
  * @section intro Introduction
@@ -125,7 +129,9 @@ vbi_event_enable(vbi_decoder *vbi, int mask)
  * @param handler Event handler function.
  * @param user_data Pointer passed to the handler.
  * 
- * @deprecated Use vbi_event_handler_register() in new code.
+ * @deprecated
+ * Replaces all existing handlers with this @a handler function,
+ * ignoring @a user_data. Use vbi_event_handler_register() in new code.
  * 
  * @return
  * FALSE on failure.
@@ -188,7 +194,9 @@ vbi_event_handler_add(vbi_decoder *vbi, int event_mask,
  * @param vbi Initialized vbi decoding context.
  * @param handler Event handler function.
  * 
- * @deprecated Use vbi_event_handler_register() in new code.
+ * @deprecated
+ * This functions lacks a user_data parameter.
+ * Use vbi_event_handler_register() in new code.
  */
 void
 vbi_event_handler_remove(vbi_decoder *vbi, vbi_event_handler handler)
@@ -833,31 +841,3 @@ vbi_version			(unsigned int *		major,
 	if (minor) *minor = VBI_VERSION_MINOR;
 	if (micro) *micro = VBI_VERSION_MICRO;
 }
-
-/**
- * @internal
- * libzvbi internal helper function.
- * Note asprintf() is a GNU libc extension.
- */
-void
-vbi_asprintf(char **errstr, char *templ, ...)
-{
-	char buf[512];
-	va_list ap;
-	int temp;
-
-	if (!errstr)
-		return;
-
-	temp = errno;
-
-	va_start(ap, templ);
-
-	vsnprintf(buf, sizeof(buf) - 1, templ, ap);
-
-	va_end(ap);
-
-	*errstr = strdup(buf);
-
-	errno = temp;
-}
