@@ -18,11 +18,13 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: caption.c,v 1.5 2002/07/16 00:13:57 mschimek Exp $ */
+/* $Id: caption.c,v 1.8 2005/01/19 04:23:53 mschimek Exp $ */
 
 #undef NDEBUG
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -35,11 +37,8 @@
 #include <X11/Xlib.h>
 #include <X11/keysym.h>
 #include <X11/Xutil.h>
-//#include <X11/xpm.h>
 
-#include <libzvbi.h>
-
-#define printable(c) ((((c) & 0x7F) < 0x20 || ((c) & 0x7F) > 0x7E) ? '.' : ((c) & 0x7F))
+#include "src/libzvbi.h"
 
 vbi_decoder *		vbi;
 vbi_pgno		pgno = -1;
@@ -445,7 +444,7 @@ sample_stream(void)
 			if (index == 1 || index == 7) {
 				printf(" %3d %02x %02x %c%c\n", s->line,
 				       s->data[0] & 0x7F, s->data[1] & 0x7F,
-				       printable(s->data[0]), printable(s->data[1]));
+				       vbi_printable (s->data[0]), vbi_printable (s->data[1]));
 			} else {
 				printf(" %3d %d ignored\n", s->line, index);
 				s--;
