@@ -18,11 +18,13 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: capture.c,v 1.9 2004/04/03 00:08:34 mschimek Exp $ */
+/* $Id: capture.c,v 1.15 2005/01/19 04:23:53 mschimek Exp $ */
 
 #undef NDEBUG
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -34,10 +36,28 @@
 #include <getopt.h>
 #endif
 
-#include <libzvbi.h>
+#include "src/libzvbi.h"
+
+vbi_inline int
+vbi_printable			(int			c)
+{
+	if (c < 0)
+		return '?';
+
+	c &= 0x7F;
+
+	if (c < 0x20 || c >= 0x7F)
+		return '.';
+
+	return c;
+}
+
+#define TEST 1
+#include "src/dvb_mux.h"
 
 vbi_capture *		cap;
 vbi_raw_decoder *	par;
+vbi_dvb_mux *		mx;
 vbi_bool		quit;
 int			src_w, src_h;
 
@@ -49,36 +69,23 @@ int			dump_wss;
 int			dump_vps;
 int			dump_sliced;
 int			bin_sliced;
+int			bin_pes;
+int			bin_ts;
 int			do_read = TRUE;
 int			do_sim;
+int			ignore_error;
 
 #include "sim.c"
 
+extern void
+vbi_capture_set_log_fp		(vbi_capture *		capture,
+				 FILE *			fp);
+extern vbi_bool vbi_capture_force_read_mode;
+
 /*
  *  Dump
  */
 
-static inline int
-odd_parity(uint8_t c)
-{
-	c ^= (c >> 4);
-	c ^= (c >> 2);
-	c ^= (c >> 1);
-
-	return c & 1;
-}
-
-extern const int8_t vbi_hamm8val[256];
-extern const uint8_t vbi_bit_reverse[256];
-
-static inline int
-vbi_hamm16(uint8_t *p)
-{
-	return vbi_hamm8val[p[0]] | (vbi_hamm8val[p[1]] << 4);
-}
-
-#define printable(c) ((((c) & 0x7F) < 0x20 || ((c) & 0x7F) > 0x7E) ? '.' : ((c) & 0x7F))
-
 #define PIL(day, mon, hour, min) \
 	(((day) << 15) + ((mon) << 11) + ((hour) << 6) + ((min) << 0))
 
@@ -121,7 +128,7 @@ decode_vps(uint8_t *buf)
 
 	printf("\nVPS:\n");
 
-	c = vbi_bit_reverse[buf[1]];
+	c = vbi_rev8 (buf[1]);
 
 	if ((int8_t) c < 0) {
 		label[l] = 0;
@@ -131,7 +138,7 @@ decode_vps(uint8_t *buf)
 
 	c &= 0x7F;
 
-	label[l] = printable(c);
+	label[l] = vbi_printable (c);
 
 	l = (l + 1) % 16;
 
@@ -161,7 +168,7 @@ decode_ttx(uint8_t *buf, int line)
 	int magazine, packet;
 	int j;
 
-	packet_address = vbi_hamm16(buf + 0);
+	packet_address = vbi_unham16p (buf + 0);
 
 	if (packet_address < 0)
 		return; /* hamming error */
@@ -173,7 +180,7 @@ decode_ttx(uint8_t *buf, int line)
 		printf("WST %x %02d %03d >", magazine, packet, line);
 
 		for (j = 0; j < 42; j++) {
-			char c = printable(buf[j]);
+			char c = vbi_printable (buf[j]);
 
 			putchar(c);
 		}
@@ -189,11 +196,11 @@ decode_xds(uint8_t *buf)
 	if (dump_xds) {
 		char c;
 
-		c = odd_parity(buf[0]) ? buf[0] & 0x7F : '?';
-		c = printable(c);
+		c = vbi_unpar8 (buf[0]);
+		c = vbi_printable (c);
 		putchar(c);
-		c = odd_parity(buf[1]) ? buf[1] & 0x7F : '?';
-		c = printable(c);
+		c = vbi_unpar8 (buf[1]);
+		c = vbi_printable (c);
 		putchar(c);
 		fflush(stdout);
 	}
@@ -207,7 +214,8 @@ decode_caption(uint8_t *buf, int line)
 
 	if (line >= 280) { /* field 2 */
 		/* 0x01xx..0x0Exx ASCII_or_NUL[0..32] 0x0Fchks */
-		if (odd_parity(buf[0]) && (c >= 0x01 && c <= 0x0F)) {
+		if (vbi_unpar8 (buf[0]) >= 0
+		    && (c >= 0x01 && c <= 0x0F)) {
 			decode_xds(buf);
 			xds_transport = (c != 0x0F);
 		} else if (xds_transport) {
@@ -218,11 +226,11 @@ decode_caption(uint8_t *buf, int line)
 	}
 
 	if (dump_cc) {
-		c = odd_parity(buf[0]) ? buf[0] & 0x7F : '?';
-		c = printable(c);
+		c = vbi_unpar8 (buf[0]);
+		c = vbi_printable (c);
 		putchar(c);
-		c = odd_parity(buf[1]) ? buf[1] & 0x7F : '?';
-		c = printable(c);
+		c = vbi_unpar8 (buf[1]);
+		c = vbi_printable (c);
 		putchar(c);
 		fflush(stdout);
 	}
@@ -245,7 +253,7 @@ decode_wss_625(uint8_t *buf)
 		"none",
 		"in active image area",
 		"out of active image area",
-		"?"
+		"<invalid>"
 	};
 	int g1 = buf[0] & 15;
 	int parity;
@@ -259,19 +267,20 @@ decode_wss_625(uint8_t *buf)
 		printf("WSS PAL: ");
 		if (!(parity & 1))
 			printf("<parity error> ");
-		printf("%s; %s mode; %s colour coding;\n"
-		       "      %s helper; reserved b7=%d; %s\n"
-		       "      open subtitles: %s; %scopyright %s; copying %s\n",
-		       formats[g1],
-		       (buf[0] & 0x10) ? "film" : "camera",
-		       (buf[0] & 0x20) ? "MA/CP" : "standard",
-		       (buf[0] & 0x40) ? "modulated" : "no",
-		       !!(buf[0] & 0x80),
-		       (buf[1] & 0x01) ? "have TTX subtitles; " : "",
-		       subtitles[(buf[1] >> 1) & 3],
-		       (buf[1] & 0x08) ? "surround sound; " : "",
-		       (buf[1] & 0x10) ? "asserted" : "unknown",
-		       (buf[1] & 0x20) ? "restricted" : "not restricted");
+		printf ("%s; %s mode; %s colour coding; %s helper; "
+			"reserved b7=%d; %s Teletext subtitles; "
+			"open subtitles: %s; %s surround sound; "
+			"copyright %s; copying %s\n",
+			formats[g1],
+			(buf[0] & 0x10) ? "film" : "camera",
+			(buf[0] & 0x20) ? "MA/CP" : "standard",
+			(buf[0] & 0x40) ? "modulated" : "no",
+			!!(buf[0] & 0x80),
+			(buf[1] & 0x01) ? "have" : "no",
+			subtitles[(buf[1] >> 1) & 3],
+			(buf[1] & 0x08) ? "have" : "no",
+			(buf[1] & 0x10) ? "asserted" : "unknown",
+			(buf[1] & 0x20) ? "restricted" : "not restricted");
 	}
 }
 
@@ -313,7 +322,7 @@ decode_sliced(vbi_sliced *s, double time, int lines)
 			putchar(' ');
 
 			for (j = 0; j < sizeof(q->data); j++) {
-				char c = printable(q->data[j]);
+				char c = vbi_printable (q->data[j]);
 				putchar(c);
 			}
 
@@ -384,6 +393,16 @@ binary_sliced(vbi_sliced *s, double time, int lines)
 	}
 }
 
+static vbi_bool
+binary_ts_pes			(vbi_dvb_mux *		mx,
+				 void *			user_data,
+				 const uint8_t *	packet,
+				 unsigned int		packet_size)
+{
+	fwrite (packet, 1, packet_size, stdout);
+	return TRUE;
+}
+
 static void
 mainloop(void)
 {
@@ -394,6 +413,7 @@ mainloop(void)
 	vbi_capture_buffer *sliced_buffer;
 	double timestamp;
 	struct timeval tv;
+	int64_t pts;
 
 	tv.tv_sec = 2;
 	tv.tv_usec = 0;
@@ -417,46 +437,58 @@ mainloop(void)
 					     &sliced_buffer, &tv);
 		}
 
+		if (0) {
+			fputc ('.', stdout);
+			fflush (stdout);
+		}
+ 
 		switch (r) {
 		case -1:
-			fprintf(stderr, "VBI read error: %d, %s\n",
-				errno, strerror(errno));
-			exit(EXIT_FAILURE);
+			fprintf(stderr, "VBI read error: %d, %s%s\n",
+				errno, strerror(errno),
+				ignore_error ? " (ignored)" : "");
+			if (ignore_error)
+				continue;
+			else
+				exit(EXIT_FAILURE);
 		case 0: 
-			fprintf(stderr, "VBI read timeout\n");
-			exit(EXIT_FAILURE);
+			fprintf(stderr, "VBI read timeout%s\n",
+				ignore_error ? " (ignored)" : "");
+			if (ignore_error)
+				continue;
+			else
+				exit(EXIT_FAILURE);
 		case 1:
 			break;
 		default:
 			assert(!"reached");
 		}
 
-		if (do_read) {
-			if (dump)
-				decode_sliced(sliced, timestamp, lines);
-			if (bin_sliced)
-				binary_sliced(sliced, timestamp, lines);
-		} else {
-			if (dump)
-				decode_sliced(sliced_buffer->data,
-					      sliced_buffer->timestamp,
-					      sliced_buffer->size
-					      / sizeof(vbi_sliced));
-			if (bin_sliced)
-				binary_sliced(sliced_buffer->data,
-					      sliced_buffer->timestamp,
-					      sliced_buffer->size
-					      / sizeof(vbi_sliced));
+		if (!do_read) {
+			sliced = sliced_buffer->data;
+			lines = sliced_buffer->size / sizeof (vbi_sliced);
+			timestamp = sliced_buffer->timestamp;
+		}
+
+		if (dump)
+			decode_sliced(sliced, timestamp, lines);
+		if (bin_sliced)
+			binary_sliced(sliced, timestamp, lines);
+		if (bin_pes || bin_ts) {
+			/* XXX shouldn't use system time. */
+			pts = timestamp * 90000;
+			_vbi_dvb_mux_mux(mx, pts, sliced, lines, -1);
 		}
 	}
 }
 
-static const char short_options[] = "d:lnpstv";
+static const char short_options[] = "123de:lnpstvPT";
 
 #ifdef HAVE_GETOPT_LONG
 static const struct option
 long_options[] = {
 	{ "device",	required_argument,	NULL,		'd' },
+	{ "ignore-error", no_argument,		NULL,		'e' },
 	{ "pid",	required_argument,	NULL,		'i' },
 	{ "dump-ttx",	no_argument,		NULL,		't' },
 	{ "dump-xds",	no_argument,		&dump_xds,	TRUE },
@@ -464,12 +496,17 @@ long_options[] = {
 	{ "dump-wss",	no_argument,		&dump_wss,	TRUE },
 	{ "dump-vps",	no_argument,		&dump_vps,	TRUE },
 	{ "dump-sliced",no_argument,		&dump_sliced,	TRUE },
+	{ "pes",	no_argument,		NULL,		'P' },
 	{ "sliced",	no_argument,		NULL,		'l' },
+	{ "ts",		no_argument,		NULL,		'T' },
 	{ "read",	no_argument,		&do_read,	TRUE },
 	{ "pull",	no_argument,		&do_read,	FALSE },
 	{ "sim",	no_argument,		NULL,		's' },
 	{ "ntsc",	no_argument,		NULL,		'n' },
 	{ "pal",	no_argument,		NULL,		'p' },
+	{ "v4l",	no_argument,		NULL,		'1' },
+	{ "v4l2-read",	no_argument,		NULL,		'2' },
+	{ "v4l2-mmap",	no_argument,		NULL,		'3' },
 	{ "verbose",	no_argument,		NULL,		'v' },
 	{ 0, 0, 0, 0 }
 };
@@ -485,17 +522,30 @@ main(int argc, char **argv)
 	char *errstr;
 	unsigned int services;
 	int scanning = 625;
-	vbi_bool verbose = FALSE;
+	int strict;
+	int verbose = 0;
 	int c, index;
+	int interface = 0;
 
 	while ((c = getopt_long(argc, argv, short_options,
 				long_options, &index)) != -1)
 		switch (c) {
 		case 0: /* set flag */
 			break;
+		case '2':
+			/* Preliminary hack for tests. */
+			vbi_capture_force_read_mode = TRUE;
+			/* fall through */
+		case '1':
+		case '3':
+			interface = c - '0';
+			break;
 		case 'd':
 			dev_name = optarg;
 			break;
+		case 'e':
+			ignore_error ^= TRUE;
+			break;
 		case 'i':
 			pid = atoi (optarg);
 			break;
@@ -508,14 +558,20 @@ main(int argc, char **argv)
 		case 'p':
 			scanning = 625;
 			break;
+		case 'P':
+			bin_pes ^= TRUE;
+			break;
 		case 's':
 			do_sim ^= TRUE;
 			break;
 		case 't':
 			dump_ttx ^= TRUE;
 			break;
+		case 'T':
+			bin_ts ^= TRUE;
+			break;
 		case 'v':
-			verbose ^= TRUE;
+			++verbose;
 			break;
 		default:
 			fprintf(stderr, "Unknown option\n");
@@ -530,6 +586,8 @@ main(int argc, char **argv)
 		| VBI_SLICED_CAPTION_625 | VBI_SLICED_VPS
 		| VBI_SLICED_WSS_625 | VBI_SLICED_WSS_CPR1204;
 
+	strict = 0;
+
 	if (do_sim) {
 		par = init_sim (scanning, services);
 	} else {
@@ -538,9 +596,9 @@ main(int argc, char **argv)
 				cap = vbi_capture_dvb_new (dev_name,
 							   scanning,
 							   &services,
-							   /* strict */ -1,
+							   strict,
 							   &errstr,
-							   verbose);
+							   !!verbose);
 				if (cap) {
 					vbi_capture_dvb_filter (cap, pid);
 					break;
@@ -554,47 +612,57 @@ main(int argc, char **argv)
 				exit(EXIT_FAILURE);
 			}
 
-			cap = vbi_capture_v4l2_new (dev_name,
-						    /* buffers */ 5,
-						    &services,
-						    /* strict */ -1,
-						    &errstr,
-						    /* trace */ verbose);
-			if (cap)
-				break;
+			if (1 != interface) {
+				cap = vbi_capture_v4l2_new (dev_name,
+							    /* buffers */ 5,
+							    &services,
+							    strict,
+							    &errstr,
+							    /* trace */
+							    !!verbose);
+				if (cap)
+					break;
 
-			fprintf (stderr, "Cannot capture vbi data "
-				 "with v4l2 interface:\n%s\n", errstr);
+				fprintf (stderr, "Cannot capture vbi data "
+					 "with v4l2 interface:\n%s\n", errstr);
 
-			free (errstr);
+				free (errstr);
+			}
 
-			cap = vbi_capture_v4l_new (dev_name,
-						   scanning,
-						   &services,
-						   /* strict */ -1,
-						   &errstr,
-						   /* trace */ verbose);
-			if (cap)
-				break;
+			if (interface < 2) {
+				cap = vbi_capture_v4l_new (dev_name,
+							   scanning,
+							   &services,
+							   strict,
+							   &errstr,
+							   /* trace */
+							   !!verbose);
+				if (cap)
+					break;
 
-			fprintf (stderr, "Cannot capture vbi data "
-				 "with v4l interface:\n%s\n", errstr);
+				fprintf (stderr, "Cannot capture vbi data "
+					 "with v4l interface:\n%s\n", errstr);
 
-			free (errstr);
+				free (errstr);
+			}
 
-			cap = vbi_capture_bktr_new (dev_name,
-						    scanning,
-						    &services,
-						    /* strict */ -1,
-						    &errstr,
-						    /* trace */ verbose);
-			if (cap)
-				break;
+			/* BSD interface */
+			if (1) {
+				cap = vbi_capture_bktr_new (dev_name,
+							    scanning,
+							    &services,
+							    strict,
+							    &errstr,
+							    /* trace */
+							    !!verbose);
+				if (cap)
+					break;
 
-			fprintf (stderr, "Cannot capture vbi data "
-				 "with bktr interface:\n%s\n", errstr);
+				fprintf (stderr, "Cannot capture vbi data "
+					 "with bktr interface:\n%s\n", errstr);
 
-			free (errstr);
+				free (errstr);
+			}
 
 			exit(EXIT_FAILURE);
 		} while (0);
@@ -602,12 +670,33 @@ main(int argc, char **argv)
 		assert ((par = vbi_capture_parameters(cap)));
 	}
 
+	if (verbose > 1) {
+		vbi_capture_set_log_fp (cap, stderr);
+	}
+
 	if (-1 == pid)
 		assert (par->sampling_format == VBI_PIXFMT_YUV420);
 
 	src_w = par->bytes_per_line / 1;
 	src_h = par->count[0] + par->count[1];
 
+	if (bin_pes) {
+		mx = _vbi_dvb_mux_pes_new (/* data_identifier */ 0x10,
+					   /* packet_size */ 8 * 184,
+					   VBI_VIDEOSTD_SET_625_50,
+					   binary_ts_pes,
+					   /* user_data */ NULL);
+		assert (NULL != mx);
+	} else if (bin_ts) {
+		mx = _vbi_dvb_mux_ts_new (/* pid */ 999,
+					  /* data_identifier */ 0x10,
+					  /* packet_size */ 8 * 184,
+					  VBI_VIDEOSTD_SET_625_50,
+					  binary_ts_pes,
+					  /* user_data */ NULL);
+		assert (NULL != mx);
+	}
+
 	mainloop();
 
 	if (!do_sim)
