@@ -18,11 +18,13 @@
  *  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-/* $Id: export.c,v 1.5 2002/11/30 02:37:18 mschimek Exp $ */
+/* $Id: export.c,v 1.10 2004/12/31 06:05:20 mschimek Exp $ */
 
 #undef NDEBUG
 
-#include "config.h"
+#ifdef HAVE_CONFIG_H
+#  include "config.h"
+#endif
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -30,13 +32,15 @@
 #include <assert.h>
 #include <unistd.h>
 
-#include <libzvbi.h>
+#include "src/libzvbi.h"
 
 vbi_decoder *		vbi;
 vbi_bool		quit = FALSE;
 vbi_pgno		pgno;
 vbi_export *		ex;
 char *			extension;
+int			cr;
+vbi_dvb_demux *		dx;
 
 static void
 handler(vbi_event *ev, void *unused)
@@ -44,7 +48,8 @@ handler(vbi_event *ev, void *unused)
 	FILE *fp;
 	vbi_page page;
 
-	fprintf(stderr, "\rPage %03x.%02x ",
+	fprintf(stderr, "%cPage %03x.%02x ",
+		cr,
 		ev->ev.ttx_page.pgno,
 		ev->ev.ttx_page.subno & 0xFF);
 
@@ -91,6 +96,40 @@ handler(vbi_event *ev, void *unused)
 		quit = TRUE;
 }
 
+static void
+pes_mainloop			(void)
+{
+	uint8_t buffer[2048];
+
+	while (1 == fread (buffer, sizeof (buffer), 1, stdin)) {
+		const uint8_t *bp;
+		unsigned int left;
+
+		bp = buffer;
+		left = sizeof (buffer);
+
+		while (left > 0) {
+			vbi_sliced sliced[64];
+			unsigned int lines;
+			int64_t pts;
+
+			lines = vbi_dvb_demux_cor (dx,
+						   sliced, 64,
+						   &pts,
+						   &bp, &left);
+			if (lines > 0) {
+				vbi_decode (vbi, sliced, lines,
+					    pts / 90000.0);
+			}
+
+			if (quit)
+				return;
+		}
+	}
+
+	fprintf(stderr, "\rEnd of stream, page %03x not found\n", pgno);
+}
+
 static void
 stream(void)
 {
@@ -166,6 +205,7 @@ main(int argc, char **argv)
 {
 	char *module, *t;
 	vbi_export_info *xi;
+	int c;
 
 	if (argc < 3) {
 		fprintf(stderr, "Usage: %s module[;options] pgno <vbi data >file\n"
@@ -179,6 +219,7 @@ main(int argc, char **argv)
 		exit(EXIT_FAILURE);
 	}
 
+	cr = isatty (STDERR_FILENO) ? '\r' : '\n';
 
 	module = argv[1];
 	pgno = strtol(argv[2], NULL, 16);
@@ -198,7 +239,17 @@ main(int argc, char **argv)
 
 	assert(vbi_event_handler_add(vbi, VBI_EVENT_TTX_PAGE, handler, NULL)); 
 
-	stream();
+	c = getchar ();
+	ungetc (c, stdin);
+
+	if (0 == c) {
+		dx = vbi_dvb_pes_demux_new (/* callback */ NULL, NULL);
+		assert (NULL != dx);
+
+		pes_mainloop ();
+	} else {
+		stream ();
+	}
 
 	exit(EXIT_SUCCESS);
 }
